'''
# cdk-library-certbot

A CDK Construct Library to automate the creation and renewal of Let's Encrypt certificates.

## Features

* Creates a lambda function that utilizes Certbot to request a certificate from Let's Encrypt
* Uploads the resulting certificate data to S3 for later retrieval
* Imports the certificate to AWS Certificate Manager for tracking expiration
* Creates a trigger to re-run and re-new if the cert will expire in the next 30 days (customizable)

## API Doc

See [API](API.md)

## References

Original [gist](# Modified from original gist https://gist.github.com/arkadiyt/5d764c32baa43fc486ca16cb8488169a) that was modified for the Lambda code

## Examples

### Typescript

```
import * as cdk from '@aws-cdk/core';
import { Certbot } from '@renovosolutions/cdk-library-certbot';
import { Architecture } from '@aws-cdk/aws-lambda';

export class CdkExampleCertsStack extends cdk.Stack {
  constructor(scope: cdk.Construct, id: string, props?: cdk.StackProps) {
    super(scope, id, props);

    let domains = [
      'example.com',
      'www.example.com'
    ]

    new Certbot(this, 'cert', {
      letsencryptDomains: domains.join(','),
      letsencryptEmail: 'webmaster+letsencrypt@example.com',
      hostedZoneNames: [
        'example.com'
      ],
      architecture: Architecture.ARM_64
    })
  }
}

```
'''
import abc
import builtins
import datetime
import enum
import typing

import jsii
import publication
import typing_extensions

from ._jsii import *

import aws_cdk.aws_events
import aws_cdk.aws_lambda
import aws_cdk.aws_s3
import aws_cdk.aws_sns
import aws_cdk.core


class Certbot(
    aws_cdk.core.Construct,
    metaclass=jsii.JSIIMeta,
    jsii_type="@renovosolutions/cdk-library-certbot.Certbot",
):
    def __init__(
        self,
        scope: aws_cdk.core.Construct,
        id: builtins.str,
        props: "ICertbotProps",
    ) -> None:
        '''
        :param scope: -
        :param id: -
        :param props: -
        '''
        jsii.create(self.__class__, self, [scope, id, props])

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="handler")
    def handler(self) -> aws_cdk.aws_lambda.Function:
        return typing.cast(aws_cdk.aws_lambda.Function, jsii.get(self, "handler"))


@jsii.interface(jsii_type="@renovosolutions/cdk-library-certbot.ICertbotProps")
class ICertbotProps(typing_extensions.Protocol):
    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="hostedZoneNames")
    def hosted_zone_names(self) -> typing.List[builtins.str]:
        '''Any additional Lambda layers to use with the created function.

        For example Lambda Extensions
        '''
        ...

    @hosted_zone_names.setter
    def hosted_zone_names(self, value: typing.List[builtins.str]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="letsencryptDomains")
    def letsencrypt_domains(self) -> builtins.str:
        '''The comma delimited list of domains for which the Let's Encrypt certificate will be valid.

        Primary domain should be first.
        '''
        ...

    @letsencrypt_domains.setter
    def letsencrypt_domains(self, value: builtins.str) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="letsencryptEmail")
    def letsencrypt_email(self) -> builtins.str:
        '''The email to associate with the Let's Encrypt certificate request.'''
        ...

    @letsencrypt_email.setter
    def letsencrypt_email(self, value: builtins.str) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="bucket")
    def bucket(self) -> typing.Optional[aws_cdk.aws_s3.Bucket]:
        '''The S3 bucket to place the resulting certificates in.

        If no bucket is given one will be created automatically.
        '''
        ...

    @bucket.setter
    def bucket(self, value: typing.Optional[aws_cdk.aws_s3.Bucket]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="enableInsights")
    def enable_insights(self) -> typing.Optional[builtins.bool]:
        '''Whether or not to enable Lambda Insights.'''
        ...

    @enable_insights.setter
    def enable_insights(self, value: typing.Optional[builtins.bool]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="insightsARN")
    def insights_arn(self) -> typing.Optional[builtins.str]:
        '''Insights layer ARN for your region.

        Defaults to US-EAST-1
        '''
        ...

    @insights_arn.setter
    def insights_arn(self, value: typing.Optional[builtins.str]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="layers")
    def layers(self) -> typing.Optional[typing.List[aws_cdk.aws_lambda.ILayerVersion]]:
        '''Hosted zone names that will be required for DNS verification with certbot.'''
        ...

    @layers.setter
    def layers(
        self,
        value: typing.Optional[typing.List[aws_cdk.aws_lambda.ILayerVersion]],
    ) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="objectPrefix")
    def object_prefix(self) -> typing.Optional[builtins.str]:
        '''The prefix to apply to the final S3 key name for the certificates.

        Default is no prefix.
        '''
        ...

    @object_prefix.setter
    def object_prefix(self, value: typing.Optional[builtins.str]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="preferredChain")
    def preferred_chain(self) -> typing.Optional[builtins.str]:
        '''Set the preferred certificate chain.

        Default None.
        '''
        ...

    @preferred_chain.setter
    def preferred_chain(self, value: typing.Optional[builtins.str]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="reIssueDays")
    def re_issue_days(self) -> typing.Optional[jsii.Number]:
        '''The numbers of days left until the prior cert expires before issuing a new one.

        Default is 30 days
        '''
        ...

    @re_issue_days.setter
    def re_issue_days(self, value: typing.Optional[jsii.Number]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="runOnDeploy")
    def run_on_deploy(self) -> typing.Optional[builtins.bool]:
        '''Whether or not to schedule a trigger to run the function after each deployment.'''
        ...

    @run_on_deploy.setter
    def run_on_deploy(self, value: typing.Optional[builtins.bool]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="runOnDeployWaitMinutes")
    def run_on_deploy_wait_minutes(self) -> typing.Optional[jsii.Number]:
        '''How many minutes to wait before running the post deployment Lambda trigger.

        Defaults to 10 minutes
        '''
        ...

    @run_on_deploy_wait_minutes.setter
    def run_on_deploy_wait_minutes(self, value: typing.Optional[jsii.Number]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="schedule")
    def schedule(self) -> typing.Optional[aws_cdk.aws_events.Schedule]:
        '''The schedule for the certificate check trigger.

        Defaults to once every Sunday.
        '''
        ...

    @schedule.setter
    def schedule(self, value: typing.Optional[aws_cdk.aws_events.Schedule]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="snsTopic")
    def sns_topic(self) -> typing.Optional[aws_cdk.aws_sns.Topic]:
        '''The SNS topic to notify when a new cert is issued.'''
        ...

    @sns_topic.setter
    def sns_topic(self, value: typing.Optional[aws_cdk.aws_sns.Topic]) -> None:
        ...

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="timeout")
    def timeout(self) -> typing.Optional[aws_cdk.core.Duration]:
        '''The timeout duration for Lambda function.'''
        ...

    @timeout.setter
    def timeout(self, value: typing.Optional[aws_cdk.core.Duration]) -> None:
        ...


class _ICertbotPropsProxy:
    __jsii_type__: typing.ClassVar[str] = "@renovosolutions/cdk-library-certbot.ICertbotProps"

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="hostedZoneNames")
    def hosted_zone_names(self) -> typing.List[builtins.str]:
        '''Any additional Lambda layers to use with the created function.

        For example Lambda Extensions
        '''
        return typing.cast(typing.List[builtins.str], jsii.get(self, "hostedZoneNames"))

    @hosted_zone_names.setter
    def hosted_zone_names(self, value: typing.List[builtins.str]) -> None:
        jsii.set(self, "hostedZoneNames", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="letsencryptDomains")
    def letsencrypt_domains(self) -> builtins.str:
        '''The comma delimited list of domains for which the Let's Encrypt certificate will be valid.

        Primary domain should be first.
        '''
        return typing.cast(builtins.str, jsii.get(self, "letsencryptDomains"))

    @letsencrypt_domains.setter
    def letsencrypt_domains(self, value: builtins.str) -> None:
        jsii.set(self, "letsencryptDomains", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="letsencryptEmail")
    def letsencrypt_email(self) -> builtins.str:
        '''The email to associate with the Let's Encrypt certificate request.'''
        return typing.cast(builtins.str, jsii.get(self, "letsencryptEmail"))

    @letsencrypt_email.setter
    def letsencrypt_email(self, value: builtins.str) -> None:
        jsii.set(self, "letsencryptEmail", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="bucket")
    def bucket(self) -> typing.Optional[aws_cdk.aws_s3.Bucket]:
        '''The S3 bucket to place the resulting certificates in.

        If no bucket is given one will be created automatically.
        '''
        return typing.cast(typing.Optional[aws_cdk.aws_s3.Bucket], jsii.get(self, "bucket"))

    @bucket.setter
    def bucket(self, value: typing.Optional[aws_cdk.aws_s3.Bucket]) -> None:
        jsii.set(self, "bucket", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="enableInsights")
    def enable_insights(self) -> typing.Optional[builtins.bool]:
        '''Whether or not to enable Lambda Insights.'''
        return typing.cast(typing.Optional[builtins.bool], jsii.get(self, "enableInsights"))

    @enable_insights.setter
    def enable_insights(self, value: typing.Optional[builtins.bool]) -> None:
        jsii.set(self, "enableInsights", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="insightsARN")
    def insights_arn(self) -> typing.Optional[builtins.str]:
        '''Insights layer ARN for your region.

        Defaults to US-EAST-1
        '''
        return typing.cast(typing.Optional[builtins.str], jsii.get(self, "insightsARN"))

    @insights_arn.setter
    def insights_arn(self, value: typing.Optional[builtins.str]) -> None:
        jsii.set(self, "insightsARN", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="layers")
    def layers(self) -> typing.Optional[typing.List[aws_cdk.aws_lambda.ILayerVersion]]:
        '''Hosted zone names that will be required for DNS verification with certbot.'''
        return typing.cast(typing.Optional[typing.List[aws_cdk.aws_lambda.ILayerVersion]], jsii.get(self, "layers"))

    @layers.setter
    def layers(
        self,
        value: typing.Optional[typing.List[aws_cdk.aws_lambda.ILayerVersion]],
    ) -> None:
        jsii.set(self, "layers", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="objectPrefix")
    def object_prefix(self) -> typing.Optional[builtins.str]:
        '''The prefix to apply to the final S3 key name for the certificates.

        Default is no prefix.
        '''
        return typing.cast(typing.Optional[builtins.str], jsii.get(self, "objectPrefix"))

    @object_prefix.setter
    def object_prefix(self, value: typing.Optional[builtins.str]) -> None:
        jsii.set(self, "objectPrefix", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="preferredChain")
    def preferred_chain(self) -> typing.Optional[builtins.str]:
        '''Set the preferred certificate chain.

        Default None.
        '''
        return typing.cast(typing.Optional[builtins.str], jsii.get(self, "preferredChain"))

    @preferred_chain.setter
    def preferred_chain(self, value: typing.Optional[builtins.str]) -> None:
        jsii.set(self, "preferredChain", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="reIssueDays")
    def re_issue_days(self) -> typing.Optional[jsii.Number]:
        '''The numbers of days left until the prior cert expires before issuing a new one.

        Default is 30 days
        '''
        return typing.cast(typing.Optional[jsii.Number], jsii.get(self, "reIssueDays"))

    @re_issue_days.setter
    def re_issue_days(self, value: typing.Optional[jsii.Number]) -> None:
        jsii.set(self, "reIssueDays", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="runOnDeploy")
    def run_on_deploy(self) -> typing.Optional[builtins.bool]:
        '''Whether or not to schedule a trigger to run the function after each deployment.'''
        return typing.cast(typing.Optional[builtins.bool], jsii.get(self, "runOnDeploy"))

    @run_on_deploy.setter
    def run_on_deploy(self, value: typing.Optional[builtins.bool]) -> None:
        jsii.set(self, "runOnDeploy", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="runOnDeployWaitMinutes")
    def run_on_deploy_wait_minutes(self) -> typing.Optional[jsii.Number]:
        '''How many minutes to wait before running the post deployment Lambda trigger.

        Defaults to 10 minutes
        '''
        return typing.cast(typing.Optional[jsii.Number], jsii.get(self, "runOnDeployWaitMinutes"))

    @run_on_deploy_wait_minutes.setter
    def run_on_deploy_wait_minutes(self, value: typing.Optional[jsii.Number]) -> None:
        jsii.set(self, "runOnDeployWaitMinutes", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="schedule")
    def schedule(self) -> typing.Optional[aws_cdk.aws_events.Schedule]:
        '''The schedule for the certificate check trigger.

        Defaults to once every Sunday.
        '''
        return typing.cast(typing.Optional[aws_cdk.aws_events.Schedule], jsii.get(self, "schedule"))

    @schedule.setter
    def schedule(self, value: typing.Optional[aws_cdk.aws_events.Schedule]) -> None:
        jsii.set(self, "schedule", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="snsTopic")
    def sns_topic(self) -> typing.Optional[aws_cdk.aws_sns.Topic]:
        '''The SNS topic to notify when a new cert is issued.'''
        return typing.cast(typing.Optional[aws_cdk.aws_sns.Topic], jsii.get(self, "snsTopic"))

    @sns_topic.setter
    def sns_topic(self, value: typing.Optional[aws_cdk.aws_sns.Topic]) -> None:
        jsii.set(self, "snsTopic", value)

    @builtins.property # type: ignore[misc]
    @jsii.member(jsii_name="timeout")
    def timeout(self) -> typing.Optional[aws_cdk.core.Duration]:
        '''The timeout duration for Lambda function.'''
        return typing.cast(typing.Optional[aws_cdk.core.Duration], jsii.get(self, "timeout"))

    @timeout.setter
    def timeout(self, value: typing.Optional[aws_cdk.core.Duration]) -> None:
        jsii.set(self, "timeout", value)

# Adding a "__jsii_proxy_class__(): typing.Type" function to the interface
typing.cast(typing.Any, ICertbotProps).__jsii_proxy_class__ = lambda : _ICertbotPropsProxy


__all__ = [
    "Certbot",
    "ICertbotProps",
]

publication.publish()
